@@ -1,25 +1,26 @@
-/*******************************************************************************
-    Copyright (c) 2016-2021 NVIDIA Corporation
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 
-    Permission is hereby granted, free of charge, to any person obtaining a copy
-    of this software and associated documentation files (the "Software"), to
-    deal in the Software without restriction, including without limitation the
-    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
-    sell copies of the Software, and to permit persons to whom the Software is
-    furnished to do so, subject to the following conditions:
-
-        The above copyright notice and this permission notice shall be
-        included in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-    DEALINGS IN THE SOFTWARE.
-
-*******************************************************************************/
 #include "linux_nvswitch.h"
 
 #include <linux/version.h>
@@ -30,6 +31,7 @@
 #include "nvCpuUuid.h"
 #include "nv-time.h"
 #include "nvlink_caps.h"
+#include "nvlink_proto.h"
 
 #include <linux/module.h>
 #include <linux/interrupt.h>
@@ -48,7 +50,7 @@
 
 #include "ioctl_nvswitch.h"
 
-const static struct
+static const struct
 {
     NvlStatus status;
     int err;
@@ -94,11 +96,7 @@ nvswitch_map_status
 #define IRQF_SHARED SA_SHIRQ
 #endif
 
-#if defined(NV_FILE_HAS_INODE)
 #define NV_FILE_INODE(file) (file)->f_inode
-#else
-#define NV_FILE_INODE(file) (file)->f_dentry->d_inode
-#endif
 
 static int nvswitch_probe(struct pci_dev *, const struct pci_device_id *);
 static void nvswitch_remove(struct pci_dev *);
@@ -242,9 +240,6 @@ static long nvswitch_ctl_unlocked_ioctl(struct file *file,
 struct file_operations device_fops =
 {
     .owner = THIS_MODULE,
-#if defined(NV_FILE_OPERATIONS_HAS_IOCTL)
-    .ioctl = nvswitch_device_ioctl,
-#endif
     .unlocked_ioctl = nvswitch_device_unlocked_ioctl,
     .open    = nvswitch_device_open,
     .release = nvswitch_device_release,
@@ -254,9 +249,6 @@ struct file_operations device_fops =
 struct file_operations ctl_fops =
 {
     .owner = THIS_MODULE,
-#if defined(NV_FILE_OPERATIONS_HAS_IOCTL)
-    .ioctl = nvswitch_ctl_ioctl,
-#endif
     .unlocked_ioctl = nvswitch_ctl_unlocked_ioctl,
 };
 
@@ -577,6 +569,8 @@ nvswitch_deinit_device
     NVSWITCH_DEV *nvswitch_dev
 )
 {
+    nvswitch_deinit_i2c_adapters(nvswitch_dev);
+
     nvswitch_lib_disable_interrupts(nvswitch_dev->lib_device);
 
     nvswitch_shutdown_device_interrupt(nvswitch_dev);
@@ -1007,6 +1001,8 @@ nvswitch_ctl_get_devices_v2(NVSWITCH_GET_DEVICES_V2_PARAMS *p)
                                                  &p->info[index].deviceState,
                                                  &p->info[index].deviceReason,
                                                  &p->info[index].driverState);
+
+            p->info[index].bTnvlEnabled = nvswitch_lib_is_tnvl_enabled(nvswitch_dev->lib_device);
             mutex_unlock(&nvswitch_dev->device_mutex);
         }
         index++;
@@ -1455,16 +1451,12 @@ nvswitch_remove
 
     list_del(&nvswitch_dev->list_node);
 
-    nvswitch_deinit_i2c_adapters(nvswitch_dev);
-
-    WARN_ON(!list_empty(&nvswitch_dev->i2c_adapter_list));
-
-    pci_set_drvdata(pci_dev, NULL);
-
     nvswitch_deinit_background_tasks(nvswitch_dev);
 
     nvswitch_deinit_device(nvswitch_dev);
 
+    pci_set_drvdata(pci_dev, NULL);
+
     pci_iounmap(pci_dev, nvswitch_dev->bar0);
 
     pci_release_regions(pci_dev);
@@ -1808,12 +1800,12 @@ nvswitch_exit
         return;
     }
 
-    nvswitch_procfs_exit();
-
     nvswitch_ctl_exit();
 
     pci_unregister_driver(&nvswitch_pci_driver);
 
+    nvswitch_procfs_exit();
+
     cdev_del(&nvswitch.cdev);
 
     unregister_chrdev_region(nvswitch.devno, NVSWITCH_MINOR_COUNT);
@@ -1825,8 +1817,7 @@ nvswitch_exit
 
 //
 // Get current time in seconds.nanoseconds
-// In this implementation, the time is from epoch time
-// (midnight UTC of January 1, 1970)
+// In this implementation, the time is monotonic time
 //
 NvU64
 nvswitch_os_get_platform_time
@@ -1840,6 +1831,28 @@ nvswitch_os_get_platform_time
     return (NvU64) timespec64_to_ns(&ts);
 }
 
+//
+// Get current time in seconds.nanoseconds
+// In this implementation, the time is from epoch time
+// (midnight UTC of January 1, 1970).
+// This implementation cannot be used for polling loops
+// due to clock skew during system startup (bug 3302382,
+// 3297170, 3273847, 3277478, 200693329).
+// Instead, nvswitch_os_get_platform_time() is used
+// for polling loops
+//
+NvU64
+nvswitch_os_get_platform_time_epoch
+(
+    void
+)
+{
+    struct timespec64 ts;
+
+    ktime_get_real_ts64(&ts);
+    return (NvU64) timespec64_to_ns(&ts);
+}
+
 void
 nvswitch_os_print
 (
@@ -1855,11 +1868,10 @@ nvswitch_os_print
     switch (log_level)
     {
         case NVSWITCH_DBG_LEVEL_MMIO:
+        case NVSWITCH_DBG_LEVEL_NOISY:
             kern_level = KERN_DEBUG;
             break;
         case NVSWITCH_DBG_LEVEL_INFO:
-            kern_level = KERN_INFO;
-            break;
         case NVSWITCH_DBG_LEVEL_SETUP:
             kern_level = KERN_INFO;
             break;
@@ -1880,6 +1892,17 @@ nvswitch_os_print
     va_end(arglist);
 }
 
+void
+nvswitch_os_report_error
+(
+    void *os_handle,
+    NvU32 error_code,
+    const char *fmt,
+    ...
+)
+{
+}
+
 void
 nvswitch_os_override_platform
 (
@@ -1887,6 +1910,8 @@ nvswitch_os_override_platform
     NvBool *rtlsim
 )
 {
+    // Never run on RTL
+    *rtlsim = NV_FALSE;
 }
 
 NvlStatus
@@ -2140,11 +2165,11 @@ _nvswitch_to_pci_dma_direction
 )
 {
     if (direction == NVSWITCH_DMA_DIR_TO_SYSMEM)
-        return PCI_DMA_FROMDEVICE;
+        return DMA_FROM_DEVICE;
     else if (direction == NVSWITCH_DMA_DIR_FROM_SYSMEM)
-        return PCI_DMA_TODEVICE;
+        return DMA_TO_DEVICE;
     else
-        return PCI_DMA_BIDIRECTIONAL;
+        return DMA_BIDIRECTIONAL;
 }
 
 NvlStatus
@@ -2165,9 +2190,9 @@ nvswitch_os_map_dma_region
 
     dma_dir = _nvswitch_to_pci_dma_direction(direction);
 
-    *dma_handle = (NvU64)pci_map_single(pdev, cpu_addr, size, dma_dir);
+    *dma_handle = (NvU64)dma_map_single(&pdev->dev, cpu_addr, size, dma_dir);
 
-    if (pci_dma_mapping_error(pdev, *dma_handle))
+    if (dma_mapping_error(&pdev->dev, *dma_handle))
     {
         pr_err("nvidia-nvswitch: unable to create PCI DMA mapping\n");
         return -NVL_ERR_GENERIC;
@@ -2194,7 +2219,7 @@ nvswitch_os_unmap_dma_region
 
     dma_dir = _nvswitch_to_pci_dma_direction(direction);
 
-    pci_unmap_single(pdev, dma_handle, size, dma_dir);
+    dma_unmap_single(&pdev->dev, dma_handle, size, dma_dir);
 
     return NVL_SUCCESS;
 }
@@ -2211,7 +2236,7 @@ nvswitch_os_set_dma_mask
     if (!pdev)
         return -NVL_BAD_ARGS;
 
-    if (pci_set_dma_mask(pdev, DMA_BIT_MASK(dma_addr_width)))
+    if (dma_set_mask(&pdev->dev, DMA_BIT_MASK(dma_addr_width)))
         return -NVL_ERR_GENERIC;
 
     return NVL_SUCCESS;
@@ -2234,7 +2259,7 @@ nvswitch_os_sync_dma_region_for_cpu
 
     dma_dir = _nvswitch_to_pci_dma_direction(direction);
 
-    pci_dma_sync_single_for_cpu(pdev, dma_handle, size, dma_dir);
+    dma_sync_single_for_cpu(&pdev->dev, dma_handle, size, dma_dir);
 
     return NVL_SUCCESS;
 }
@@ -2256,7 +2281,7 @@ nvswitch_os_sync_dma_region_for_device
 
     dma_dir = _nvswitch_to_pci_dma_direction(direction);
 
-    pci_dma_sync_single_for_device(pdev, dma_handle, size, dma_dir);
+    dma_sync_single_for_device(&pdev->dev, dma_handle, size, dma_dir);
 
     return NVL_SUCCESS;
 }
@@ -2381,6 +2406,17 @@ nvswitch_os_strncmp
     return strncmp(s1, s2, length);
 }
 
+char*
+nvswitch_os_strncat
+(
+    char *s1,
+    const char *s2,
+    NvLength length
+)
+{
+    return strncat(s1, s2, length);
+}
+
 void *
 nvswitch_os_memset
 (
@@ -2486,26 +2522,22 @@ nvswitch_os_vsnprintf
 void
 nvswitch_os_assert_log
 (
-    int cond,
     const char *fmt,
     ...
 )
 {
-    if(cond == 0x0)
+    if (printk_ratelimit())
     {
-        if (printk_ratelimit())
-        {
-            va_list arglist;
-            char fmt_printk[NVSWITCH_LOG_BUFFER_SIZE];
+        va_list arglist;
+        char fmt_printk[NVSWITCH_LOG_BUFFER_SIZE];
 
-            va_start(arglist, fmt);
-            vsnprintf(fmt_printk, sizeof(fmt_printk), fmt, arglist);
-            va_end(arglist);
-            nvswitch_os_print(NVSWITCH_DBG_LEVEL_ERROR, fmt_printk);
-            WARN_ON(1);
-         }
-         dbg_breakpoint();
-    }
+        va_start(arglist, fmt);
+        vsnprintf(fmt_printk, sizeof(fmt_printk), fmt, arglist);
+        va_end(arglist);
+        nvswitch_os_print(NVSWITCH_DBG_LEVEL_ERROR, fmt_printk);
+        WARN_ON(1);
+     }
+     dbg_breakpoint();
 }
 
 /*
@@ -2672,3 +2704,17 @@ nvswitch_os_get_supported_register_events_params
     *os_descriptor = NV_FALSE;
     return NVL_SUCCESS;
 }
+
+NvlStatus
+nvswitch_os_get_pid
+(
+    NvU32 *pPid
+)
+{
+    if (pPid != NULL)
+    {
+        *pPid = task_pid_nr(current);
+    }
+    
+    return NVL_SUCCESS;
+}
